#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QFile>
#include <QXmlStreamReader>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    scene = new QGraphicsScene ();
    ui->graphicsView->setScene (scene);
    initPalette ();
    readFile ("../demo/data.xml");
}

MainWindow::~MainWindow()
{
    delete ui;
}

#include <QToolButton>
#include <QIcon>
#include <QDrag>
#include <QMimeData>

void ColorButton::mousePressEvent(QMouseEvent *event)
{
    QDrag * drag = new QDrag (this);
    drag->start ();
};


void MainWindow::initPalette()
{
    QStringList names;
    names << "red" << "blue" << "green" << "yellow";

    for (auto name : names)
    {
        auto b = new ColorButton;
        b->setText (name);

        QPixmap image (20, 20);
        image.fill (QColor (0, 0, 0, 0));

        QPainter painter (&image);
        QColor c = QColor (name);
        b->color = c;

        painter.setPen (c);
        painter.setBrush (c);
        painter.drawEllipse (2, 2, 16, 16);

        QIcon icon (image);
        b->setIcon (icon);

        ui->mainToolBar->addWidget (b);
    }
}

QString ParamList::text (QString name)
{
    return this->value (name, "");
}

int ParamList::number (QString name, int default_value)
{
    QString s = text (name);
    bool ok;
    int n = s.toInt (&ok);
    if (!ok) n = default_value;
    return n;
}

QGraphicsRectItem * MainWindow::createRectangle
                                (QGraphicsItem * target,
                                 ParamList & list)
{
    QGraphicsRectItem * shape = new QGraphicsRectItem;

    shape->setPos (list.number ("x", 0),
                   list.number ("y", 0));

    shape->setRect (0, 0,
                    list.number ("w", 200),
                    list.number ("h", 100));

    modifyShape (target, shape, list);
    return shape;
}

QGraphicsEllipseItem * MainWindow::createEllipse
                                   (QGraphicsItem * target,
                                    ParamList & list)
{
    QGraphicsEllipseItem * shape = new QGraphicsEllipseItem;

    shape->setPos (list.number ("x", 0),
                   list.number ("y", 0));

    shape->setRect (0, 0,
                    list.number ("w", 20),
                    list.number ("h", 20));

    modifyShape (target, shape, list);
    return shape;
}

void MainWindow::modifyShape (QGraphicsItem * target,
                              QAbstractGraphicsShapeItem * shape,
                              ParamList & list)
{
    if (list.contains ("pen"))
        shape->setPen (QColor (list.text ("pen")));
    if (list.contains ("brush"))
       shape->setBrush (QColor (list.text ("brush")));

    shape->setFlag(QGraphicsItem::ItemIsMovable);
    shape->setFlag(QGraphicsItem::ItemIsSelectable);

    if (target != NULL)
    {
        shape->setParentItem (target);
        // QPointF pos = target->pos();
        // shape->moveBy (pos.x(), pos.y());
    }
    else
        scene->addItem (shape);
}


void MainWindow::readFile (QString fileName)
{
    QFile file (fileName);
    if (file.open (QFile::ReadOnly | QFile::Text))
    {
        QTreeWidgetItem * branch = ui->treeWidget->invisibleRootItem();
        QGraphicsItem * target = NULL;

        QXmlStreamReader reader (&file);
        while (! reader.atEnd ())
        {
            if (reader.isStartElement())
            {
                QString s = reader.name().toString();
                QTreeWidgetItem * item = new QTreeWidgetItem;
                item->setText (0, s);
                item->setForeground (0, QColor ("blue"));
                branch->addChild (item);
                branch = item;

                ParamList list;
                for (QXmlStreamAttribute a : reader.attributes())
                {
                    QString t = a.name().toString();
                    QString u = a.value().toString();

                    list [t] = u;
                    // list.insert (t, u)

                    QTreeWidgetItem * item2 = new QTreeWidgetItem;
                    item2->setText (0, t + " = " + u);
                    item2->setForeground (0, QColor ("green"));
                    branch->addChild (item2);
                }

                if (s == "rectangle")
                    target = createRectangle (target, list);
                else if (s == "ellipse")
                    target = createEllipse (target, list);
            }
            else if (reader.isEndElement())
            {
                QString s = reader.name().toString();

                branch = branch->parent();

                if (s == "rectangle" || s == "ellipse")
                   target = target->parentItem ();
            }

            reader.readNext ();
        }
    }
    ui->treeWidget->expandAll();
}
